/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */
#include <stdio.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <openthread/thread.h>
#include <openthread/tasklet.h>
#include <openthread/instance.h>
#include <openthread/dataset.h>
#include <openthread/coap.h>
#include <zephyr/net/openthread.h>
#include <zephyr/net/socket.h>

#define TASK_NAME "test_task"
#define TASK_STACK_SIZE 8 * 1024
#define TASK_PRIORITY 5

#define SLEEP_TIME_MS   1000
#define LED_NODE DT_ALIAS(led3)
static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED_NODE, gpios); 

static otInstance *instance;
static char  aNetworkName[] = "XizhenElec";
static otOperationalDataset aDataset;
const char PSKD[] = "DRRLCK8";
otExtAddress *extAddr = NULL;

void CommissionerStateCb(otCommissionerState aState, void *aContext)
{
    
    switch(aState)
    {
        case OT_COMMISSIONER_STATE_DISABLED:
            printf("OT_COMMISSIONER_STATE_DISABLED\r\n");
            break;
        case OT_COMMISSIONER_STATE_PETITION:
            printf("OT_COMMISSIONER_STATE_PETITION\r\n");
            break;
        case OT_COMMISSIONER_STATE_ACTIVE:
            printf("OT_COMMISSIONER_STATE_ACTIVE\r\n");
            otError error = otCommissionerAddJoiner(instance, extAddr, PSKD, 120);
                if (error != OT_ERROR_NONE) {
                printf("Failed to add joiner: %d\r\n", error);
                }
            break;
        default:
            break;
    }
}

void CommissionerJoinerCb(otCommissionerJoinerEvent aEvent, const otJoinerInfo *aJoinerInfo, const otExtAddress *aJoinerId, void *aContext)
{
    uint8_t i;
    printf("CommissionerJoinerCb.evt = %d\r\n", aEvent);    
    switch(aEvent)
    {
        case OT_COMMISSIONER_JOINER_START:
            printf("OT_COMMISSIONER_JOINER_START\r\n");
            if(aJoinerInfo)
            {
                printf("mType = %d\r\n", aJoinerInfo->mType);
                printf("mEui64 = \r\n");
                for(i = 0; i < sizeof(aJoinerInfo->mSharedId.mEui64.m8); i++)
                    printf("%02x ", aJoinerInfo->mSharedId.mEui64.m8[i]);
                printf("\r\n");
                printf("mPSKd = %s\r\n", aJoinerInfo->mPskd.m8);
                printf("mExpirationTime = %d\r\n", aJoinerInfo->mExpirationTime);
            }
            break;
        case OT_COMMISSIONER_JOINER_CONNECTED:
            printf("OT_COMMISSIONER_JOINER_CONNECTED\r\n");
            break;
        case OT_COMMISSIONER_JOINER_FINALIZE:
            printf("OT_COMMISSIONER_JOINER_FINALIZE\r\n");
            break;
        case OT_COMMISSIONER_JOINER_END:
            printf("OT_COMMISSIONER_JOINER_END\r\n");
            break;
        case OT_COMMISSIONER_JOINER_REMOVED:
            printf("OT_COMMISSIONER_JOINER_REMOVED\r\n");
            otError error = otCommissionerAddJoiner(instance, extAddr, PSKD, 120);
            if (error != OT_ERROR_NONE) {
            printf("Failed to add joiner: %d\r\n", error);
            }
            break;
    }
    if(aJoinerId)   
    {
        printf("aJoinerId = ");
        for(i = 0; i < sizeof(aJoinerId->m8); i++)
            printf("%02x ", aJoinerId->m8[i]);
        printf("\r\n");
    }
}

void handleNetifStateChanged(uint32_t aFlags, void *aContext)
{
    if(aFlags & OT_CHANGED_THREAD_ROLE)
    {
       otDeviceRole changedRole = otThreadGetDeviceRole(aContext);

       switch (changedRole)
       {
            case OT_DEVICE_ROLE_LEADER:
                printf("OT_DEVICE_ROLE_LEADER!\r\n");
                otCoapStart(instance, 1000);
                otCommissionerStart(instance, CommissionerStateCb, CommissionerJoinerCb, NULL);
                break;
            case OT_DEVICE_ROLE_ROUTER:
                printf("OT_DEVICE_ROLE_ROUTER!\r\n");
                break;

            case OT_DEVICE_ROLE_CHILD:
                printf("OT_DEVICE_ROLE_CHILD!\r\n");
                break;

            case OT_DEVICE_ROLE_DETACHED:
                printf("OT_DEVICE_ROLE_DETACHED!\r\n");
                break;
            case OT_DEVICE_ROLE_DISABLED:
                printf("OT_DEVICE_ROLE_DISABLED!\r\n");
                break;
        }
    }
    // if(aFlags & OT_CHANGED_IP6_ADDRESS_ADDED)
    // {
    //     printf("OT_CHANGED_IP6_ADDRESS_ADDED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_IP6_ADDRESS_REMOVED)
    // {
    //     printf("OT_CHANGED_IP6_ADDRESS_REMOVED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_LL_ADDR)
    // {
    //     printf("OT_CHANGED_THREAD_LL_ADDR!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_ML_ADDR)
    // {
    //     printf("OT_CHANGED_THREAD_ML_ADDR!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_RLOC_ADDED)
    // {
    //     printf("OT_CHANGED_THREAD_RLOC_ADDED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_RLOC_REMOVED)
    // {
    //     printf("OT_CHANGED_THREAD_RLOC_REMOVED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_PARTITION_ID)
    // {
    //     printf("OT_CHANGED_THREAD_PARTITION_ID!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_KEY_SEQUENCE_COUNTER)
    // {
    //     printf("OT_CHANGED_THREAD_KEY_SEQUENCE_COUNTER!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_NETDATA)
    // {
    //     printf("OT_CHANGED_THREAD_NETDATA!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_CHILD_ADDED)
    // {
    //     printf("OT_CHANGED_THREAD_CHILD_ADDED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_CHILD_REMOVED)
    // {
    //     printf("OT_CHANGED_THREAD_CHILD_REMOVED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_IP6_MULTICAST_SUBSCRIBED)
    // {
    //     printf("OT_CHANGED_IP6_MULTICAST_SUBSCRIBED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_IP6_MULTICAST_UNSUBSCRIBED)
    // {
    //     printf("OT_CHANGED_IP6_MULTICAST_UNSUBSCRIBED!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_CHANNEL)
    // {
    //     printf("OT_CHANGED_THREAD_CHANNEL!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_PANID)
    // {
    //     printf("OT_CHANGED_THREAD_PANID!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_NETWORK_NAME)
    // {
    //     printf("OT_CHANGED_THREAD_NETWORK_NAME!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_EXT_PANID)
    // {
    //     printf("OT_CHANGED_THREAD_EXT_PANID!\r\n");
    // }
    // if(aFlags & OT_CHANGED_NETWORK_KEY)
    // {
    //     printf("OT_CHANGED_NETWORK_KEY!\r\n");
    // }
    // if(aFlags & OT_CHANGED_PSKC)
    // {
    //     printf("OT_CHANGED_PSKC!\r\n");
    // }
    // if(aFlags & OT_CHANGED_SECURITY_POLICY)
    // {
    //     printf("OT_CHANGED_SECURITY_POLICY!\r\n");
    // }
    // if(aFlags & OT_CHANGED_CHANNEL_MANAGER_NEW_CHANNEL)
    // {
    //     printf("OT_CHANGED_CHANNEL_MANAGER_NEW_CHANNEL!\r\n");
    // }
    // if(aFlags & OT_CHANGED_SUPPORTED_CHANNEL_MASK)
    // {
    //     printf("OT_CHANGED_SUPPORTED_CHANNEL_MASK!\r\n");
    // }
    // if(aFlags & OT_CHANGED_COMMISSIONER_STATE)
    // {
    //     printf("OT_CHANGED_COMMISSIONER_STATE!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_NETIF_STATE)
    // {
    //     printf("OT_CHANGED_THREAD_NETIF_STATE!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_BACKBONE_ROUTER_STATE)
    // {
    //     printf("OT_CHANGED_THREAD_BACKBONE_ROUTER_STATE!\r\n");
    // }
    // if(aFlags & OT_CHANGED_THREAD_BACKBONE_ROUTER_LOCAL)
    // {
    //     printf("OT_CHANGED_THREAD_BACKBONE_ROUTER_LOCAL!\r\n");
    // }
    // if(aFlags & OT_CHANGED_JOINER_STATE)
    // {
    //     printf("OT_CHANGED_JOINER_STATE!\r\n");
    // }
    // if(aFlags & OT_CHANGED_ACTIVE_DATASET)
    // {
    //     printf("OT_CHANGED_ACTIVE_DATASET!\r\n");
    // }
    // if(aFlags & OT_CHANGED_PENDING_DATASET)
    // {
    //     printf("OT_CHANGED_PENDING_DATASET!\r\n");
    // }
    // if(aFlags & OT_CHANGED_NAT64_TRANSLATOR_STATE)
    // {
    //     printf("OT_CHANGED_NAT64_TRANSLATOR_STATE!\r\n");
    // }
    // if(aFlags & OT_CHANGED_PARENT_LINK_QUALITY)
    // {
    //     printf("OT_CHANGED_PARENT_LINK_QUALITY!\r\n");
    // }
}

void setNetworkConfiguration(otInstance *aInstance)
{
    memset(&aDataset, 0, sizeof(otOperationalDataset));

    /*
     * Fields that can be configured in otOperationDataset to override defaults:
     *     Network Name, Mesh Local Prefix, Extended PAN ID, PAN ID, Delay Timer,
     *     Channel, Channel Mask Page 0, Network Key, PSKc, Security Policy
     */
    aDataset.mActiveTimestamp.mSeconds             = 1;
    aDataset.mActiveTimestamp.mTicks               = 1;
    aDataset.mActiveTimestamp.mAuthoritative       = true;
    aDataset.mComponents.mIsActiveTimestampPresent = true;

    aDataset.mDelay = 1;
    aDataset.mComponents.mIsDelayPresent = true;

    /* Set Channel to 20 */
    aDataset.mChannel                      = 20;
    aDataset.mComponents.mIsChannelPresent = true;


    /* Set Pan ID to 2222 */
    aDataset.mPanId                      = (otPanId)0x2222;
    aDataset.mComponents.mIsPanIdPresent = true;

    /* Set Extended Pan ID to C0DE1AB5C0DE1AB5 */
    uint8_t extPanId[OT_EXT_PAN_ID_SIZE] = {0xC0, 0xDE, 0x1A, 0xB5, 0xC0, 0xDE, 0x1A, 0xB5};
    memcpy(aDataset.mExtendedPanId.m8, extPanId, sizeof(aDataset.mExtendedPanId));
    aDataset.mComponents.mIsExtendedPanIdPresent = true;

    /* Set network key to 1234C0DE1AB51234C0DE1AB51234C0DE */
    uint8_t key[OT_NETWORK_KEY_SIZE] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff};
    memcpy(aDataset.mNetworkKey.m8, key, sizeof(aDataset.mNetworkKey));
    aDataset.mComponents.mIsNetworkKeyPresent = true;

    /* Set Network Name to OTCodelab */
    size_t length = strlen(aNetworkName);
    assert(length <= OT_NETWORK_NAME_MAX_SIZE);
    memcpy(aDataset.mNetworkName.m8, aNetworkName, length);
    aDataset.mComponents.mIsNetworkNamePresent = true;

    otDatasetGeneratePskc("eeee", &aDataset.mNetworkName, &aDataset.mExtendedPanId, &aDataset.mPskc);
    aDataset.mComponents.mIsPskcPresent = true;

    aDataset.mSecurityPolicy.mRotationTime = 0;
    aDataset.mSecurityPolicy.mObtainNetworkKeyEnabled = false;
    aDataset.mSecurityPolicy.mNativeCommissioningEnabled = true;
    aDataset.mSecurityPolicy.mRoutersEnabled = true;
    aDataset.mSecurityPolicy.mExternalCommissioningEnabled = false;
    aDataset.mSecurityPolicy.mCommercialCommissioningEnabled = false;
    aDataset.mSecurityPolicy.mAutonomousEnrollmentEnabled = true;
    aDataset.mSecurityPolicy.mNetworkKeyProvisioningEnabled = true;
    aDataset.mSecurityPolicy.mTobleLinkEnabled = false;
    aDataset.mSecurityPolicy.mNonCcmRoutersEnabled = false;
    aDataset.mSecurityPolicy.mVersionThresholdForRouting = 0;
    aDataset.mComponents.mIsSecurityPolicyPresent = true;
   
    otDatasetSetActive(aInstance, &aDataset);
}

static void task_entry_point(void *, void *, void *)
{
	printf("Task run!\r\n");
    printf("OpenThread Version: %s\r\n", otGetVersionString());
	instance = otInstanceInitSingle();
    printf("Instance Initialized: %d\r\n", otInstanceIsInitialized(instance));
    printf("Instance Radio Version: %s\r\n", otGetRadioVersionString(instance));
    printf("Instance ID: %d\r\n", otInstanceGetId(instance));
    otSetStateChangedCallback(instance, handleNetifStateChanged, instance);
    setNetworkConfiguration(instance);
    otIp6SetEnabled(instance, true);
    otThreadSetEnabled(instance, true);
    printf("OpenThread Start!\r\n");
    
    while (1)
    {
        otTaskletsProcess(instance);
    }
    otInstanceFinalize(instance);
}

K_THREAD_DEFINE(id, TASK_STACK_SIZE, task_entry_point, NULL, NULL, NULL, TASK_PRIORITY, 0, 1000);

int main(void)
{
	gpio_is_ready_dt(&led);
	gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);

	// k_thread_start(id);

	while (1) 
	{
		gpio_pin_toggle_dt(&led);
		k_msleep(SLEEP_TIME_MS / 4);
	}

	return 0;
}
